@@ -4,21 +4,20 @@
 
 use std::io::Write;
 
-use syn;
+use syn::ext::IdentExt;
 
 use crate::bindgen::config::{Config, Language, LayoutConfig};
 use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
 use crate::bindgen::dependencies::Dependencies;
-use crate::bindgen::ir::SynFieldHelpers;
 use crate::bindgen::ir::{
-    AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, Item, ItemContainer, Path,
-    Repr, ReprAlign, ReprStyle, ToCondition, Type,
+    AnnotationSet, Cfg, ConditionWrite, Documentation, Field, GenericParams, Item, ItemContainer,
+    Path, Repr, ReprAlign, ReprStyle, ToCondition, Type,
 };
 use crate::bindgen::library::Library;
 use crate::bindgen::mangle;
 use crate::bindgen::monomorph::Monomorphs;
 use crate::bindgen::rename::{IdentifierType, RenameRule};
-use crate::bindgen::utilities::{find_first_some, IterHelpers};
+use crate::bindgen::utilities::IterHelpers;
 use crate::bindgen::writer::{ListType, Source, SourceWriter};
 
 #[derive(Debug, Clone)]
@@ -26,7 +25,7 @@ pub struct Union {
     pub path: Path,
     pub export_name: String,
     pub generic_params: GenericParams,
-    pub fields: Vec<(String, Type, Documentation)>,
+    pub fields: Vec<Field>,
     pub tuple_union: bool,
     pub alignment: Option<ReprAlign>,
     pub cfg: Option<Cfg>,
@@ -50,14 +49,14 @@ impl Union {
             layout_config.ensure_safe_to_represent(&align)?;
         }
 
-        let path = Path::new(item.ident.to_string());
+        let path = Path::new(item.ident.unraw().to_string());
 
         let (fields, tuple_union) = {
             let out = item
                 .fields
                 .named
                 .iter()
-                .try_skip_map(|x| x.as_ident_and_type(&path))?;
+                .try_skip_map(|field| Field::load(field, &path))?;
             (out, false)
         };
 
@@ -73,10 +72,11 @@ impl Union {
         ))
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         path: Path,
         generic_params: GenericParams,
-        fields: Vec<(String, Type, Documentation)>,
+        fields: Vec<Field>,
         alignment: Option<ReprAlign>,
         tuple_union: bool,
         cfg: Option<Cfg>,
@@ -89,17 +89,17 @@ impl Union {
             export_name,
             generic_params,
             fields,
-            alignment,
             tuple_union,
+            alignment,
             cfg,
             annotations,
             documentation,
         }
     }
 
-    pub fn simplify_standard_types(&mut self) {
-        for &mut (_, ref mut ty, _) in &mut self.fields {
-            ty.simplify_standard_types();
+    pub fn simplify_standard_types(&mut self, config: &Config) {
+        for field in &mut self.fields {
+            field.ty.simplify_standard_types(config);
         }
     }
 
@@ -114,14 +114,14 @@ impl Union {
             return;
         }
 
-        for &(_, ref ty, _) in &self.fields {
-            ty.add_monomorphs(library, out);
+        for field in &self.fields {
+            field.ty.add_monomorphs(library, out);
         }
     }
 
     pub fn mangle_paths(&mut self, monomorphs: &Monomorphs) {
-        for &mut (_, ref mut ty, _) in &mut self.fields {
-            ty.mangle_paths(monomorphs);
+        for field in &mut self.fields {
+            field.ty.mangle_paths(monomorphs);
         }
     }
 }
@@ -156,58 +156,68 @@ impl Item for Union {
     }
 
     fn resolve_declaration_types(&mut self, resolver: &DeclarationTypeResolver) {
-        for &mut (_, ref mut ty, _) in &mut self.fields {
-            ty.resolve_declaration_types(resolver);
+        for field in &mut self.fields {
+            field.ty.resolve_declaration_types(resolver);
         }
     }
 
     fn rename_for_config(&mut self, config: &Config) {
         config.export.rename(&mut self.export_name);
-        for &mut (_, ref mut ty, _) in &mut self.fields {
-            ty.rename_for_config(config, &self.generic_params);
+        for field in &mut self.fields {
+            field.ty.rename_for_config(config, &self.generic_params);
         }
 
-        let rules = [
-            self.annotations.parse_atom::<RenameRule>("rename-all"),
-            config.structure.rename_fields,
-        ];
+        let rules = self
+            .annotations
+            .parse_atom::<RenameRule>("rename-all")
+            .unwrap_or(config.structure.rename_fields);
 
         if let Some(o) = self.annotations.list("field-names") {
             let mut overriden_fields = Vec::new();
 
-            for (i, &(ref name, ref ty, ref doc)) in self.fields.iter().enumerate() {
+            for (i, field) in self.fields.iter().enumerate() {
                 if i >= o.len() {
-                    overriden_fields.push((name.clone(), ty.clone(), doc.clone()));
+                    overriden_fields.push(field.clone());
                 } else {
-                    overriden_fields.push((o[i].clone(), ty.clone(), doc.clone()));
+                    overriden_fields.push(Field {
+                        name: o[i].clone(),
+                        ty: field.ty.clone(),
+                        cfg: field.cfg.clone(),
+                        annotations: field.annotations.clone(),
+                        documentation: field.documentation.clone(),
+                    });
                 }
             }
 
             self.fields = overriden_fields;
-        } else if let Some(r) = find_first_some(&rules) {
+        } else if let Some(r) = rules.not_none() {
             self.fields = self
                 .fields
                 .iter()
-                .map(|x| {
-                    (
-                        r.apply(&x.0, IdentifierType::StructMember),
-                        x.1.clone(),
-                        x.2.clone(),
-                    )
+                .map(|field| Field {
+                    name: r
+                        .apply(&field.name, IdentifierType::StructMember)
+                        .into_owned(),
+                    ty: field.ty.clone(),
+                    cfg: field.cfg.clone(),
+                    annotations: field.annotations.clone(),
+                    documentation: field.documentation.clone(),
                 })
                 .collect();
         } else if self.tuple_union {
             // If we don't have any rules for a tuple union, prefix them with
             // an underscore so it still compiles
-            for &mut (ref mut name, ..) in &mut self.fields {
-                name.insert(0, '_');
+            for field in &mut self.fields {
+                field.name.insert(0, '_');
             }
         }
     }
 
     fn add_dependencies(&self, library: &Library, out: &mut Dependencies) {
-        for &(_, ref ty, _) in &self.fields {
-            ty.add_dependencies_ignoring_generics(&self.generic_params, library, out);
+        for field in &self.fields {
+            field
+                .ty
+                .add_dependencies_ignoring_generics(&self.generic_params, library, out);
         }
     }
 
@@ -239,7 +249,7 @@ impl Item for Union {
         let mangled_path = mangle::mangle_path(
             &self.path,
             generic_values,
-            library.get_config().export.mangle_separator.as_deref(),
+            &library.get_config().export.mangle,
         );
 
         let monomorph = Union::new(
@@ -247,7 +257,13 @@ impl Item for Union {
             GenericParams::default(),
             self.fields
                 .iter()
-                .map(|x| (x.0.clone(), x.1.specialize(&mappings), x.2.clone()))
+                .map(|field| Field {
+                    name: field.name.clone(),
+                    ty: field.ty.specialize(&mappings),
+                    cfg: field.cfg.clone(),
+                    annotations: field.annotations.clone(),
+                    documentation: field.documentation.clone(),
+                })
                 .collect(),
             self.alignment,
             self.tuple_union,
@@ -256,10 +272,7 @@ impl Item for Union {
             self.documentation.clone(),
         );
 
-        // Instantiate any monomorphs for any generic paths we may have just created.
-        monomorph.add_monomorphs(library, out);
-
-        out.insert_union(self, monomorph, generic_values.to_owned());
+        out.insert_union(library, self, monomorph, generic_values.to_owned());
     }
 }
 
@@ -279,28 +292,33 @@ impl Source for Union {
         //   typedef union {
         // C with Both as style:
         //   typedef union Name {
-        if config.language == Language::C && config.style.generate_typedef() {
-            out.write("typedef ");
+        match config.language {
+            Language::C if config.style.generate_typedef() => out.write("typedef "),
+            Language::C | Language::Cxx => {}
+            Language::Cython => out.write(config.style.cython_def()),
         }
 
         out.write("union");
 
-        if let Some(align) = self.alignment {
-            match align {
-                ReprAlign::Packed => {
-                    if let Some(ref anno) = config.layout.packed {
-                        write!(out, " {}", anno);
+        // Cython supports `packed` on structs (see comments there), but not on unions.
+        if config.language != Language::Cython {
+            if let Some(align) = self.alignment {
+                match align {
+                    ReprAlign::Packed => {
+                        if let Some(ref anno) = config.layout.packed {
+                            write!(out, " {}", anno);
+                        }
                     }
-                }
-                ReprAlign::Align(n) => {
-                    if let Some(ref anno) = config.layout.aligned_n {
-                        write!(out, " {}({})", anno, n);
+                    ReprAlign::Align(n) => {
+                        if let Some(ref anno) = config.layout.aligned_n {
+                            write!(out, " {}({})", anno, n);
+                        }
                     }
                 }
             }
         }
 
-        if config.language == Language::Cxx || config.style.generate_tag() {
+        if config.language != Language::C || config.style.generate_tag() {
             write!(out, " {}", self.export_name);
         }
 
@@ -312,15 +330,9 @@ impl Source for Union {
             out.new_line();
         }
 
-        if config.documentation {
-            out.write_vertical_source_list(&self.fields, ListType::Cap(";"));
-        } else {
-            let vec: Vec<_> = self
-                .fields
-                .iter()
-                .map(|&(ref name, ref ty, _)| (name.clone(), ty.clone()))
-                .collect();
-            out.write_vertical_source_list(&vec[..], ListType::Cap(";"));
+        out.write_vertical_source_list(&self.fields, ListType::Cap(";"));
+        if config.language == Language::Cython && self.fields.is_empty() {
+            out.write("pass");
         }
 
         // Emit the post_body section, if relevant
